"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const url_1 = require("url");
const unist_util_visit_1 = __importDefault(require("unist-util-visit"));
const mdast_util_to_string_1 = __importDefault(require("mdast-util-to-string"));
const github_slugger_1 = __importDefault(require("github-slugger"));
const githubConstants_1 = require("./githubConstants");
const permalinkIconAst_1 = __importDefault(require("./permalinkIconAst"));
const ABSOLUTE_URL = /^(https?:\/\/|\/\/)/i;
const SITE_URL = /^(https?:\/\/|\/\/)nextjs\.org/i;
// The headers will be updated to include a link to their hash
const HEADINGS = ['h2', 'h3', 'h4', 'h5', 'h6'];
function removeExt(path) {
    const basePath = path.split(/#|\?/)[0];
    const { dir, name } = path_1.parse(basePath);
    return path_1.join(dir, name) + path.substring(basePath.length);
}
function visitCard(node) {
    if (!node.children ||
        !node.properties ||
        !node.properties.className ||
        !node.properties.className.includes('card'))
        return;
    const anchor = node.children.find((n) => n.tagName === 'a');
    if (!anchor || !anchor.children)
        return;
    const title = anchor.children.find((n) => n.tagName === 'b');
    const text = anchor.children.find((n) => n.tagName === 'small');
    if (!title || !text)
        return;
    const titleText = title.children.pop();
    // Remove `:` from the title if it ends with it
    if (titleText && titleText.value && titleText.value.endsWith(':')) {
        titleText.value = titleText.value.slice(0, -1);
    }
    title.children.push(titleText);
    anchor.children = [{ ...title, tagName: 'h4' }, text];
}
function rehypeDocs({ filePath, tag }) {
    const slugger = new github_slugger_1.default();
    const anchorSlugger = new github_slugger_1.default();
    // Don't use the custom tag here, relative URLs to repo files should always go to canary
    const blobUrl = `${githubConstants_1.GITHUB_URL}/${githubConstants_1.REPO_NAME}/blob/canary`;
    function visitAnchor(node) {
        const props = node.properties;
        const href = props === null || props === void 0 ? void 0 : props.href;
        if (!href)
            return;
        props.href = href.replace(SITE_URL, '');
        const isDocs = href.startsWith('/docs');
        if (props.href === href) {
            const isAbsoluteUrl = ABSOLUTE_URL.test(href);
            const isHash = href[0] === '#';
            const isRepoUrl = !isHash && !isDocs;
            if (isAbsoluteUrl || isRepoUrl) {
                props.className = 'absolute';
                props.target = '_blank';
                props.rel = 'noopener noreferrer';
                if (!isAbsoluteUrl) {
                    // Turn any relative URL that's not handled by the Next.js site into an absolute URL
                    props.href = blobUrl + url_1.resolve(filePath, href);
                }
                return;
            }
        }
        const [relativePath, hash] = props.href.split('#');
        // Reset the slugger because single pages can have multiple urls to the same hash
        anchorSlugger.reset();
        // The URL is relative at this point
        props.className = 'relative';
        // Update the hash used by anchors to match the one set for headers
        props.href = hash
            ? `${relativePath}#${anchorSlugger.slug(hash)}`
            : relativePath;
        // Relative URL for another documentation route
        if (isDocs) {
            props.href = removeExt(tag ? props.href.replace('/docs', `/docs/tag/${tag}`) : props.href);
        }
    }
    function visitHeading(node) {
        const text = mdast_util_to_string_1.default(node);
        if (!text)
            return;
        const id = slugger.slug(text);
        node.properties.className = 'heading';
        node.children = [
            {
                type: 'element',
                tagName: 'span',
                properties: { id },
            },
            {
                type: 'element',
                tagName: 'a',
                properties: {
                    href: `#${id}`,
                },
                children: node.children,
            },
            {
                type: 'element',
                tagName: 'span',
                properties: { className: 'permalink' },
                children: [permalinkIconAst_1.default],
            },
        ];
    }
    return function transformer(tree) {
        unist_util_visit_1.default(tree, (node) => node.tagName === 'a', visitAnchor);
        unist_util_visit_1.default(tree, (node) => HEADINGS.includes(node.tagName), visitHeading);
        unist_util_visit_1.default(tree, (node) => node.tagName === 'div', visitCard);
    };
}
exports.default = rehypeDocs;
//# sourceMappingURL=rehypeDocs.js.map